#include "allen.h"
#include "allenop.h"
#include <stdlib.h>

/* print rel to string */
char *sprrel(char *s, RELTYPE r)
{

  *s = '\0';

  strcat(s,"( ");
  if (r&REQ)
    strcat(s,"= ");
  if (r&RLT)
    strcat(s,"< ");
  if (r&RGT)
    strcat(s,"> ");
  if (r&RD)
    strcat(s,"d ");
  if (r&RDI)
    strcat(s,"di ");
  if (r&RO)
    strcat(s,"o ");
  if (r&ROI)
    strcat(s,"oi ");
  if (r&RM)
    strcat(s,"m ");
  if (r&RMI)
    strcat(s,"mi ");
  if (r&RS)
    strcat(s,"s ");
  if (r&RSI)
    strcat(s,"si ");
  if (r&RF)
    strcat(s,"f ");
  if (r&RFI)
    strcat(s,"fi ");
  strcat(s,")");
  return(s);
}



/* print constraint to string */
char *sprcons(char *s, RELTYPE r, int node1, int node2)
{
  char relstr[100];

  sprintf(s,"%2d %2d ",node1,node2);
  sprrel(relstr,r);
  strcat(s,relstr);
}

/* print an entire csp to outfile */
void printcsp(FILE *outfile, int maxnodeid, char *typeid, RELTYPE csp[MAXCSP][MAXCSP])
{
  int i,j;
  char line[80];

  fprintf(outfile,"%d %s\n",maxnodeid,typeid);
  for (i = 0; i <= maxnodeid; i++) 
    for (j = i+1; j <= maxnodeid; j++) 
      if (csp[i][j] != DALL) {
	sprcons(line, csp[i][j], i, j);
	fprintf(outfile,"%s\n",line);
      }
  fprintf(outfile,".\n");
}
  
/*parse rel from string */
int parserel(char *line, RELTYPE *r)
{
  RELTYPE result;
  char *oldline;
  
  oldline = line;
  result = 0;


  while ((*line != '\0') && (*line != '(')) line++;
  if (*(line++) != '(') {
    fprintf(stderr,"\n*** no '(' in rel spec: '%s'\n",oldline);
    return(0);
  }
  while (*line == ' ') line++;
  while ((*line != ')' ) && *line) {
    while (*line == ' ') line++;
    switch (*(line++)) {
    case '=' : result |= REQ; break;
    case '<' : result |= RLT; break;
    case '>' : result |= RGT; break;
    case 'd' : if (*line == 'i') {
      result |= RDI; line++;
    } else result |= RD;
      break;
    case 'o' : if (*line == 'i') {
      result |= ROI; line++;
    } else result |= RO;
      break;
    case 'm' : if (*line == 'i') {
      result |= RMI; line++;
    } else result |= RM;
      break;
    case 's' : if (*line == 'i') {
      result |= RSI; line++;
    } else result |= RS;
      break;
    case 'f' : if (*line == 'i') {
      result |= RFI; line++;
    } else result |= RF;
      break;
      default : fprintf(stderr,"\n*** Illegal rel spec: '%s'\n",oldline);
      return(0);
    }
    while (*line == ' ') line++;
  }
  if (*line != ')') {
    fprintf(stderr,"\n*** Missing ')' in rel spec in '%s'\n",oldline);
    return(0);
  }		
  *r = result;
  return(1);
}


/* parse constraint  from string */
int parsecons(char *line, RELTYPE *r, int *node1, int *node2)
{

  *node1 = -1;
  *node2 = -1;

  if (sscanf(line,"%d %d",node1,node2) != 2) {
    fprintf(stderr,"\n*** no node ids in rel spec: '%s'\n",line);
    return(0);
  }

  return(parserel(line, r));
}

int parsecsp(FILE *infile, int limit, int *maxnodeid, char *typeid, RELTYPE csp[MAXCSP][MAXCSP])
{
  char line[128];
  int i,j;
  RELTYPE r;

  if (feof(infile)) return(0);
  while (fgets(line,128,infile) && ((line[0] == '\n') || (line[0] == '\0')));
  if (feof(infile)) return(0);

  if (sscanf(line,"%d %s",maxnodeid,typeid) != 2) {
    fprintf(stderr,"\n*** Header line of CSP not found: '%s'\n",line);
    return(-1);
  }
  
  if (*maxnodeid > limit) {
    fprintf(stderr,"\n*** Maxnode too large: '%s'\n",line);
    return(-1);
  }

  for (i = 0; i <= *maxnodeid; i++)
    for (j = i+1; j <= *maxnodeid; j++)
      csp[i][j] = DALL;

  do {
    fgets(line,128,infile);
    if (line[0] == '.') return (1);
    if (parsecons(line, &r, &i, &j)) {
      if ((i > *maxnodeid) || (i < 0) || (j > *maxnodeid) || (j < 0) ||
	  (i == j)) {
	fprintf(stderr,"\n*** Illegal node id: '%s'\n",line);
	return(-1);
      }
      if (i > j) csp[j][i] &= inverse(r);
      else csp[i][j] &= r;
    } else return(-1);
  } while (1);
}

/* parse alg and store as set */
int parsealg(FILE *infile, char alg[MAXSET])
{ 
  char line[100];
  RELTYPE r;
  int size;

  
  size = 0;
  for (r=0; r<MAXSET; r++) alg[r] = 0;

  while (fgets(line,100,infile)) {
    if ((line[0] == '.') || (line[0] == '\0')) return(size);
    if (!parserel(line,&r)) return(0);
    alg[r] = 1;
    size++;
  }
  return(size);
}
