#include "allen.h"

/* Basic composition table */
RELTYPE	comp[MAXREL][MAXREL] = {
/*EQ*/	{REQ,RLT,RGT,RD,RDI,RO,
	 ROI,RM,RMI,RS,RSI,RF,RFI},
/*LT*/	{RLT, RLT, DALL,(RLT|RO|RM|RD|RS),RLT,RLT,(RLT|RO|RM|RD|RS),
	 RLT,(RLT|RO|RM|RD|RS),RLT,RLT,(RLT|RO|RM|RD|RS),RLT},
/*GT*/	{RGT,DALL,RGT,(RGT|ROI|RMI|RD|RF),RGT,(RGT|ROI|RMI|RD|RF),
	 RGT,(RGT|ROI|RMI|RD|RF),RGT,(RGT|ROI|RMI|RD|RF),RGT,RGT,RGT},
/*D*/	{RD,RLT,RGT,RD,DALL,(RLT|RO|RM|RD|RS),(RGT|ROI|RMI|RD|RF),RLT,RGT,
	 RD, (RGT|ROI|RMI|RD|RF),RD,(RLT|RO|RM|RD|RS)},
/*DI*/	{RDI,(RLT|RO|RM|RDI|RFI),(RGT|ROI|RDI|RMI|RSI),(RO|ROI|RD|RDI|REQ|RS|RSI|RF|RFI),
	 RDI,(RO|RDI|RFI),(ROI|RDI|RSI),(RO|RDI|RFI),(ROI|RDI|RSI),
	 (RO|RDI|RFI),RDI,(ROI|RDI|RSI),RDI},
/*O*/	{RO,RLT,(RGT|ROI|RDI|RMI|RSI),(RO|RD|RS),(RLT|RO|RM|RDI|RFI),
	 (RLT|RO|RM),(RO|ROI|RD|RDI|REQ|RS|RSI|RF|RFI),RLT,(ROI|RDI|RSI),RO,(RO|RDI|RFI),
	 (RO|RD|RS),(RLT|RO|RM)},
/*OI*/	{ROI,(RLT|RO|RM|RDI|RFI),RGT,(ROI|RD|RF),(RGT|ROI|RDI|RMI|RSI),
	 (RO|ROI|RD|RDI|REQ|RS|RSI|RF|RFI),(RGT|ROI|RMI),(RO|RDI|RFI),RGT,(ROI|RD|RF),
	 (RGT|ROI|RMI),ROI,(ROI|RDI|RSI)},
/*M*/	{RM,RLT,(RGT|ROI|RDI|RMI|RSI),(RO|RD|RS),RLT,RLT,(RO|RD|RS),
	 RLT,(RF|RFI|REQ),RM,RM,(RO|RD|RS),RLT},
/*MI*/	{RMI,(RLT|RO|RM|RDI|RFI),RGT,(ROI|RD|RF),RGT,(ROI|RD|RF),RGT,
	 (RS|RSI|REQ),RGT,(ROI|RD|RF),RGT,RMI,RMI},
/*S*/	{RS,RLT,RGT,RD,(RLT|RO|RM|RDI|RFI),(RLT|RO|RM),(ROI|RD|RF),RLT,
	 RMI,RS,(RS|RSI|REQ),RD,(RLT|RO|RM)},
/*SI*/	{RSI,(RLT|RO|RM|RDI|RFI),RGT,(ROI|RD|RF),RDI,(RO|RDI|RFI),ROI,
	 (RO|RDI|RFI),RMI,(RS|RSI|REQ),RSI,ROI,RDI},
/*F*/	{RF,RLT,RGT,RD,(RGT|ROI|RDI|RMI|RSI),(RO|RD|RS),(RGT|ROI|RMI),
	 RM,RGT,RD,(RGT|ROI|RMI),RF,(RF|RFI|REQ)},
/*FI*/	{RFI,RLT,(RGT|ROI|RDI|RMI|RSI),(RO|RD|RS),RDI,RO,(ROI|RSI|RDI),
	 RM,(ROI|RSI|RDI),RO,RDI,(RF|RFI|REQ),RFI}};

/* compute composition using the composition table */
RELTYPE composition(RELTYPE r1, RELTYPE r2)
{
  RELTYPE result;
  register RELTYPE x1;
  register RELTYPE x2;
  
  result = 0;
  for (x1 = 0; x1 < MAXREL; x1++) 
    if ((r1>>x1)&1)
      for (x2 = 0; x2 < MAXREL; x2++) 
	if ((r2>>x2)&1)
	  result |= comp[x1][x2];
  return(result);
}


/* compute the inverse of a relation */
RELTYPE inverse(RELTYPE r)
{
  return(((r&REQ) ? REQ : 0) |
	 ((r&RLT) ? RGT : 0) |
	 ((r&RGT) ? RLT : 0) |
	 ((r&RD)  ? RDI : 0) |
	 ((r&RDI) ? RD  : 0) |
	 ((r&RO)  ? ROI : 0) |
	 ((r&ROI) ? RO  : 0) |
	 ((r&RM)  ? RMI : 0) |
	 ((r&RMI) ? RM  : 0) |
	 ((r&RS)  ? RSI : 0) |
	 ((r&RSI) ? RS  : 0) |
	 ((r&RF)  ? RFI : 0) |
	 ((r&RFI) ? RF  : 0));
}

int relvalue(RELTYPE r)
{
  register int bytecnt;
  register int tempval;

  bytecnt = 0;
  tempval = 0;
  if (r & REQ) { 
    bytecnt++;
    tempval += 1;
  }
  if (r & RGT) {
    bytecnt++;
    tempval += 3;
  }
  if (r & RLT) {
    bytecnt++;
    tempval += 3;
  }
  if (r & RD) {
    bytecnt++;
    tempval += 4;
  }
  if (r & RDI) {
    bytecnt++;
    tempval += 3;
  }
  if (r & RO) {
    bytecnt++;
    tempval += 4;
  }
  if (r & ROI) {
    bytecnt++;
    tempval += 4;
  }
  if (r & RM) {
    bytecnt++;
    tempval += 2;
  }
  if (r & RMI) {
    bytecnt++;
    tempval += 2;
  }
  if (r & RS) {
    bytecnt++;
    tempval += 2;
  }
  if (r & RSI) {
    bytecnt++;
    tempval += 2;
  }
  if (r & RF) {
    bytecnt++;
    tempval += 2;
  }
  if (r & RFI) {
    bytecnt++;
    tempval += 2;
  }
  return(tempval);
}

