/* Select all relations that are only expressible using 3CNF formulae */

#include <stdlib.h>
#include "allen.h"
#include "allenio.h"

#define MAXDISJ 100

#define PNEQ 1
#define PGE  2
#define PLE  4
#define PGT  (PGE|PNEQ)
#define PLT  (PLE|PNEQ)
#define PEQ  (PLE|PGE)
#define PNO  (PGE|PLE|PNEQ)
#define PALL 0

int basrel[MAXREL][4] = {
/* EQ */ {PEQ,PLT,PGT,PEQ},
/* LT */ {PLT,PLT,PLT,PLT},
/* GT */ {PGT,PGT,PGT,PGT},
/* D  */ {PGT,PLT,PGT,PLT},
/* DI */ {PLT,PLT,PGT,PGT},
/* O  */ {PLT,PLT,PGT,PLT},
/* OI */ {PGT,PLT,PGT,PGT},
/* M  */ {PLT,PLT,PEQ,PLT},
/* MI */ {PGT,PEQ,PGT,PGT},
/* S  */ {PEQ,PLT,PGT,PLT},
/* SI */ {PEQ,PLT,PGT,PGT},
/* F  */ {PGT,PLT,PGT,PEQ},
/* FI */ {PLT,PLT,PGT,PEQ}};


void prvector(f,vec)
	FILE *f;
	int vec[4];
{
  int i;

  fprintf(f,"[ ");
  for (i=0;i<4;i++) 
    switch (vec[i]) {
      case PNO:  fprintf(f,"*  "); break;
      case PEQ:  fprintf(f,"=  "); break;
      case PGT:  fprintf(f,">  "); break;
      case PGE:  fprintf(f,">= "); break;
      case PLT:  fprintf(f,"<  "); break;
      case PLE:  fprintf(f,"<= "); break;
      case PNEQ: fprintf(f,"<> "); break;
      case PALL: fprintf(f,"?  "); break;
      default: fprintf(stderr,"**Illegal point relation: %d\n",vec[i]);
               exit(-1);
               break;
    }
  fprintf(f,"]");
}
 
void prrep(f,pall,r,conj,disjset)
	FILE *f;
	int pall;
	int r;
	int conj[4];
	int disjset[MAXDISJ][4];
{
  char s[100];
  register int i;

  if (pall) {
    fprintf(f,"%s = \n",sprrel(s,r));
    fprintf(f,"  AND: ");
    prvector(f,conj);
    fprintf(f,"\n");
    for (i=0; i<MAXDISJ; i++) {
        if (disjset[i][0] != -1) {
	  fprintf(f,"       OR: ");
	  prvector(f,disjset[i]);
	  fprintf(f,"\n");
	}
    }
    fprintf(f,"\n");    
  } else fprintf(f,"%s\n",sprrel(s,r));
}

/* compute strongest conjunction that satisfies the relation */
void strongcj(r,conj)
	int r;
	register int conj[4];
{
  register int br;
  register int bit;
  register int i;

  for (i=0;i<4;i++) conj[i] = PNO;
  bit = 1;
  for (br=0;br<MAXREL;br++) {
    if (r&bit) 
      for (i=0;i<4;i++)
        conj[i] &= basrel[br][i];
    bit = bit<<1;
  }
#ifdef DEBUG
  {
    char s[100];
    printf("  ** STRONGCJ: %s -> ",sprrel(s,r));
    prvector(stdout,conj);
    printf("\n");
  }
#endif
}

/* compute relation corresponding to conjunction */
int cjrel(conj)
	register int conj[4]; 
{
  register int br;
  int r;
  int res;
  int bit;
  register int i;

  r = 0;
  bit = 1;
  for (br=0;br<MAXREL;br++) {
    res = DALL;
    for (i=0;i<4;i++)
      if (basrel[br][i] != (conj[i]|basrel[br][i])) res = 0;
    r |= (res&bit);
    bit = bit << 1;
  }
#ifdef DEBUG
  {
    char s[100];
    printf("  ** CJREL: ");
    prvector(stdout,conj);
    printf(" -> %s\n",sprrel(s,r));
  }
#endif
  return(r);
}


/* compute relation corresponding to disjunction */
int djrel(disj)
	register int disj[4];
{
  register int br;
  int r;
  int res;
  register int bit;
  register int i;

  r = 0;
  bit = 1;
  for (br=0;br<MAXREL;br++) {
    res = 0;
    for (i=0;i<4;i++)
      if ((disj[i]|(basrel[br][i])) == basrel[br][i]) res = DALL;
    r |= (res&bit);
    bit = bit << 1;
  }
#ifdef DEBUG
  {
    char s[100];
    printf("  ** DJREL: ");
    prvector(stdout,disj);
    printf(" -> %s\n",sprrel(s,r));
  }
#endif
   return(r);
}    

/* test whether r1 is more constrained than r2 (or equal) */
int subrel(r1,r2)
     register int r1, r2;
{
   return ((r1|r2) == r2);
}


/* compute strongest 2CNF-clause that covers the orginal relation
   and is not covered by the strongest "conjunctive" relation
   for index pos1,pos2 (0..3) being allowed to be different from *.
   If there is such a clause, we return the corresponding relation
   and disj is set. Otherwise, DALL is returned and disj is set to PALL.
*/
int duocnfatpos(pos1,pos2,r,conjr,disj)
     int pos1, pos2, r, conjr;
     int disj[4];
{
   register int i, i1, i2;
   int rel, oldrel;
   int cdisj[4];
   
   for (i=0;i<4;i++) {
       cdisj[i] = PNO;
       disj[i] = PALL;
   }
   oldrel = DALL;
   for (i1 = 0; i1 <= PNO; i1++) 
     for (i2 = 0; i2 <= PNO; i2++) 
        if  (((i2 == PLE) || (i2 == PGE)|| (i2 == PNEQ)) &&
	     ((i1 == PLE) || (i1 == PGE)|| (i1 == PNEQ))) {
        cdisj[pos1] = i1;
	cdisj[pos2] = i2;
        rel = djrel(cdisj);
        if (subrel(r,rel) && (!subrel(conjr,rel)) &&
            subrel(rel,oldrel)) {
           oldrel = rel;
           for (i=0;i<4;i++) disj[i] = cdisj[i];
        }
      }
   return(oldrel);  
 }

/* try to find 2CNF representation for relation r 
   if there is one, return 1, otherwise return 0.
*/

int findrep(r,conj,disjset)
     int r;
     int conj[4];
     int disjset[MAXDISJ][4];
{
  register int i,j;
  register int cjr;
  int djrs[MAXDISJ];
  register tmpr;
  register int p1,p2;

  for (i=0;i<MAXDISJ;i++) {
    djrs[i] = DALL;
    disjset[i][0] = -1;
  }
  strongcj(r,conj);
  cjr = cjrel(conj);
  if (cjr == r) return(1);
  else {
    tmpr = cjr;
    for (i=0;i<MAXDISJ;i++) {
      djrs[i] = duocnfatpos(0,3,r,tmpr,disjset[i]);
      if (djrs[i] == DALL) break;
      tmpr &= djrs[i];
    }
    for (p1=0;p1<3;p1++)
      for (p2=p1+1;p2<4;p2++) 
	if ((p1 != 0) || (p2 != 3)) 
	  for (j=i;j<MAXDISJ;j++) {
	    djrs[j] = duocnfatpos(p1,p2,r,tmpr,disjset[j]);
	    if (djrs[j] == DALL) {
	      i = j;
	      break;
	    }
	    tmpr &= djrs[j];
	  }
    if (tmpr != r) return(0);
    for (i=0;i<MAXDISJ;i++) {
      if (djrs[i] == DALL) disjset[i][0] = -1;
      else {
	for (j=0;j<MAXDISJ;j++) {
	  if ((i != j) &&
	      (subrel(djrs[j],djrs[i]))) {
	    disjset[i][0] = -1;
	    djrs[i] = DALL;
	  }
	}
      }
    }
    return(1);
  }
}




void main()
{
   int r;
   int conj[4];
   int disjset[MAXDISJ][4];
   char relstr[100];
   

   for (r=1; r<MAXSET ; r++) {
     if (!findrep(r,conj,disjset)) {
       sprrel(relstr,r);
       printf("%s\n",relstr);
     }
   }
   printf(".\n");
 }
