/* reads a set of Allen CSPs and outputs info 
   on their average properties */

#include "allen.h"
#include "allenio.h"


double size = 0.0;
double degree = 0.0;
double density =0.0;
double labsize = 0.0;
double totallabsize = 0.0;
double congraphs = 0.0;
double dcongraphs = 0.0;
double tcongraphs = 0.0;
int instances = 0;
char csptype[100];

int connected(RELTYPE csp[MAXCSP][MAXCSP], int maxnode) 
{
  static int nodes[MAXCSP];
  int changed;
  register int i, j, countdown;


  countdown = maxnode;
  nodes[0] = 1;
  for (i = 1; i<=maxnode; i++) nodes[i] = 0;

  do {
    changed = 0;
    for (i=0; i<=maxnode; i++)
      for (j=i+1; j<= maxnode; j++) {
	if ((csp[i][j] != DALL) && (nodes[i] != nodes[j])) {
	  changed = 1;
	  nodes[i] = 1;
	  nodes[j] = 1;
	  if ((--countdown) == 0) return(1);
	}
      }
  } while (changed);

  for (i=0; i<=maxnode; i++)
    if (nodes[i] == 0) return(0);
  return(1);
}

int dbconnected(RELTYPE csp[MAXCSP][MAXCSP], int maxnode) 
{
  RELTYPE save;
  int i,j;
  for (i=0; i<=maxnode; i++)
    for (j=i+1; j<= maxnode; j++) {
      if (csp[i][j] != DALL) {
	save = csp[i][j];
	csp[i][j] = DALL;
	if (!connected(csp,maxnode)) {
	  csp[i][j] = save;
	  return(0);
	}
	csp[i][j] = save;
      }
    }
  return(1);
}

int trconnected(RELTYPE csp[MAXCSP][MAXCSP], int maxnode) 
{
  RELTYPE save;
  int i,j;
  for (i=0; i<=maxnode; i++)
    for (j=i+1; j<= maxnode; j++) {
      if (csp[i][j] != DALL) {
	save = csp[i][j];
	csp[i][j] = DALL;
	if (!dbconnected(csp,maxnode)) {
	  csp[i][j] = save;
	  return(0);
	}
	csp[i][j] = save;
      }
    }
  return(1);
}

int proponecsp()
{
  static RELTYPE csp[MAXCSP][MAXCSP];
  int maxnode;
  int i,j, edges, cedges, sh, b;
  double llabsize;
  double ltotallabsize;
  double ldegree;
  double ldensity;

  

  if (parsecsp(stdin, MAXCSP-1, &maxnode, csptype, csp) == 1) {
    instances++;
    size += (double)(maxnode+1);

    edges = 0;
    cedges = 0;
    ltotallabsize = 0.0;
    llabsize = 0.0;

    for (i=0; i<=maxnode; i++)
      for (j=i+1; j<= maxnode; j++) {
	edges++;
	if (csp[i][j] != DALL) {
	  cedges++;
	  sh = 1;
	  for (b=0; b<13; b++) {
	    if (sh&csp[i][j]) {
	      ltotallabsize += 1.0;
	      llabsize += 1.0;
	    }
	    sh = sh << 1;
	  }
	}
	else ltotallabsize += 13.0;
      }
    if (connected(csp, maxnode)) congraphs += 1.0;
    if (dbconnected(csp, maxnode)) dcongraphs += 1.0;
/*    if (trconnected(csp, maxnode)) tcongraphs += 1.0; */
    totallabsize += ltotallabsize/(double)edges;
    labsize += llabsize/(double)cedges;
    density += ((double)cedges)/((double)edges);
    degree  += ((double)cedges)*2.0/((double)(maxnode+1));
    return(1);
  } else return(0);
}

void main()
{
  while (proponecsp());
  printf("Summary of generated CSPs\n");
  printf("  Last CSP type:     %s\n",csptype);
  printf("  Instances:         %5d\n",instances);
  printf("  Avg. Label size:   %8.3f\n",(labsize/(double)instances));
  printf("  Avg. total Lab s.: %8.3f\n",(totallabsize/(double)instances));
  printf("  Avg. degree:       %8.3f\n",(degree/(double)instances));
  printf("  Avg. density:      %8.3f\n",(density/(double)instances));
  printf("  Connected:         %8.3f\n",congraphs/(double)instances);
  printf("  Double Connected:  %8.3f\n",dcongraphs/(double)instances);
/*  printf("  Triple Connected:  %8.3f\n",tcongraphs/(double)instances); */

  exit(0);
}

  
